/*
   Client pour invoquer le web service skybotconesearch
   Info: https://ssp.imcce.fr/webservices/skybot/
*/

#include <string.h>
#include <stdio.h>
#include <getopt.h>

#include "soapH.h"
#include "SkyBoTBinding.nsmap"

#include "help.h"
#include "convert.h"

int main(int argc, char *argv[])
{ 
   /* declaration des variables */
   const char *wsname = "http://ssp.localhost/webservices/skybot/skybot.php";
   struct soap soap;
   struct ns1__skybotConeSearchRequest input;
   struct ns1__skybotconesearchResponse output;
   char *substr;
   const char *lex = ";";
   const char *fov_sep = ";";
   const char *mimetext = "text";

   char *s_options = "d:f:hl:m:o:r:";
   static struct option long_options[] = {
      {"date",     required_argument, 0, 'd'},
      {"fov",      required_argument, 0, 'f'},
      {"help",     no_argument,       0, 'h'},
      {"observer", required_argument, 0, 'l'},
      {"mime",     required_argument, 0, 'm'},
      {"output",   required_argument, 0, 'o'},
      {"radius",   required_argument, 0, 'r'}
   };
   int c, option_index = 0;
   char *s_date = NULL;
   char *s_coord = NULL;
   char **fov = NULL;

   /* initialisations par defaut */
   input.radius = malloc(strlen("900x900") + 1);
   strcpy(input.radius, "900x900");
   input.mime = malloc(strlen("text") + 1);
   strcpy(input.mime, "text");
   input.output = malloc(strlen("basic") + 1);
   strcpy(input.output, "basic");
   input.observer = malloc(strlen("500") + 1);
   strcpy(input.observer, "500");

   /* lecture des arguments */
   while((c = getopt_long(argc, argv, s_options, long_options, &option_index)) != -1)
   {
      switch (c)
      {
      case 'h':
         usage_help(argv[0], 0);
         exit(EXIT_SUCCESS);
      case 'd':
         s_date = optarg;
         break;
      case 'f':
         s_coord = optarg;
         break;
      case 'r':
         input.radius = malloc(strlen(optarg) + 1);
         strcpy(input.radius, optarg);
         break;
      case 'm':
         input.mime = malloc(strlen(optarg) + 1);
         strcpy(input.mime, optarg);
         break;
      case 'o':
         input.output = malloc(strlen(optarg) + 1);
         strcpy(input.output, optarg);
         break;
      case 'l':
         input.observer = malloc(strlen(optarg) + 1);
         strcpy(input.observer, optarg);
         break;
      default:
         usage_help(argv[0], 0);
         exit(EXIT_SUCCESS);
      }
   }

   if(s_date != NULL)
   {
      input.epoch = malloc(strlen(s_date) + 1);
      strcpy(input.epoch, s_date);
   }
   else
   {
      usage_help(argv[0], 0);
      exit(EXIT_SUCCESS);
   }

   if(s_coord != NULL)
   {
      fov = str_split(s_coord, fov_sep);
      input.alpha = atof(fov[0]);
      input.delta = atof(fov[1]);
   }
   else
   {
      usage_help(argv[0], 0);
      exit(EXIT_SUCCESS);
   }

   /* Instanciation du web service */
   soap_init(&soap);

   /* Definition de l'entete SOAP */
   soap.header = soap_malloc(&soap,sizeof(struct SOAP_ENV__Header));
   struct ns1__clientID *p = malloc(sizeof(*p));
   p->from = "Skybot.Cclient";
   p->hostip = "";
   soap.header->ns1__clientID = p;

   /* Invocation du web service */
   if (soap_call_ns1__skybotconesearch(&soap, wsname, NULL, &input, &output) == SOAP_OK)
   {
      /* Print result */
      if (strcmp(input.mime,mimetext) == 0) {
         printf("# Flag: %d\n", output.result->flag);
         printf("# Ticket: %ld\n", output.result->ticket);
         printf("# Status-code: %d\n", output.result->status);
         substr = strtok(output.result->result, lex);
         while (substr != NULL) {
            printf("%s\n", substr);
            substr = strtok(NULL, lex);
         }
      } else {
         printf("%s\n", output.result->result);
      }
   }
   else
   {
      /* Print SOAP error */
      soap_print_fault(&soap, stderr);
   }

   free(p);
   soap_end(&soap);
   soap_done(&soap);
   return EXIT_SUCCESS;
}
