/*
   Client pour invoquer le web service skybotstatus 
   Info: https://ssp.imcce.fr/webservices/skybot/
*/

#include <string.h>
#include <stdio.h>
#include <getopt.h>

#include "soapH.h"
#include "SkyBoTBinding.nsmap"

#include "help.h"

int main(int argc, char *argv[])
{
   /* declaration des variables */
   const char *wsname = "http://ssp.localhost/webservices/skybot/skybot.php";
   struct soap soap;
   struct ns1__skybotstatusResponse output;
   char mime[8];
   char *date;
   char *project;
   char *substr;
   const char *lex = ";";
   const char *mimetext = "text";

   char *s_options = "hd:m:p:";
   static struct option long_options[] = {
      {"help",     no_argument,       0, 'h'},
      {"date",     required_argument, 0, 'd'},
      {"mime",     required_argument, 0, 'm'},
      {"project",  required_argument, 0, 'p'}
   };
   int c, option_index = 0;
   char *s_date = NULL;
   char *s_project = NULL;

   /* initialisations par defaut */
   date = malloc(1);
   strcpy(date, "");
   strcpy(mime, "text");
   project = malloc(6);
   strcpy(project, "sbot2");

   /* lecture des arguments */
   while((c = getopt_long(argc, argv, s_options, long_options, &option_index)) != -1)
   {
      switch (c)
      {
      case 'h':
         usage_help(argv[0], 2);
         exit(EXIT_SUCCESS);
      case 'd':
         s_date = optarg;
         break;
      case 'm':
         strcpy(mime, optarg);
         break;
      case 'p':
         s_project = optarg;
         break;
      default:
         usage_help(argv[0], 2);
         exit(EXIT_SUCCESS);
      }
   }

   if(s_date != NULL) {
      date = malloc(strlen(s_date) + 1);
      strcpy(date, s_date);
   }

   if(s_project != NULL) {
      project = malloc(strlen(s_project) + 1);
      strcpy(project, s_project);
   }

   /* Instanciation du web service */
   soap_init(&soap);

   /* Definition de l'entete SOAP */
   soap.header = soap_malloc(&soap,sizeof(struct SOAP_ENV__Header));
   struct ns1__clientID *p = malloc(sizeof(*p));
   p->from = "Skybot.Cclient";
   p->hostip = "";
   soap.header->ns1__clientID = p;

   /* Invocation du web service */
   if (soap_call_ns1__skybotstatus(&soap, wsname, NULL, mime, date, project, &output) == SOAP_OK)
   {
      /* Print result */
      if (strcmp(mime,mimetext) == 0) {
         printf("# Flag: %d\n", output.result->flag);
         printf("# Ticket: %ld\n", output.result->ticket);
         printf("# Status-code: %d\n", output.result->status);
         substr = strtok(output.result->result, lex);
         while( substr != NULL ) {
            printf("%s\n", substr);
            substr = strtok(NULL, lex);
         }
      } else {
         printf("%s\n", output.result->result);
      }
   } 
   else
   {
      /* Print SOAP error */
      soap_print_fault(&soap, stderr);
   }

   free(p);
   soap_end(&soap);
   soap_done(&soap);
   return EXIT_SUCCESS;
}

